<?php
/**
 * Pre-Parse Post Plugin
 * Copyright © 2009 Ryan Gordon - NCAABBS Skunkworks Division
 *
 * Website: http://ncaabbs.com
 * License: GNU/GPL
 *
 */

// Disallow direct access to this file for security reasons
if(!defined("IN_MYBB"))
{
	die("Direct initialization of this file is not allowed.<br /><br />Please make sure IN_MYBB is defined.");
}

$plugins->add_hook('newthread_do_newthread_end', 'preparserpost_datahandler_post_insert');
$plugins->add_hook('newreply_do_newreply_end', 'preparserpost_datahandler_post_insert');
$plugins->add_hook('datahandler_post_update', 'preparserpost_datahandler_post_update');
$plugins->add_hook('showthread_start', 'preparserpost_showthread');
$plugins->add_hook('postbit', 'preparserpost_functions_post');

function preparserpost_info()
{
	return array(
		"name"			=> "Pre-Parse Post",
		"description"	=> "A plugin which aims to speed up MyBB's showthread by caching parsed versions of posts",
		"website"		=> "http://thetikitiki.com",
		"author"		=> "Ryan Gordon",
		"authorsite"	=> "http://thetikitiki.com",
		"version"		=> "1.0.2",
		"compatibility" 	=> "14*",
		"guid"			=> "",
	);
}

function preparserpost_install()
{
	global $db, $parser, $mybb, $lang;
	
	@set_time_limit(0);
	
	if(!$db->table_exists("parsedposts"))
	{
		$db->query("CREATE TABLE ".TABLE_PREFIX."parsedposts (
			pid int NOT NULL,
			parsedpost text NOT NULL,
			tid int NOT NULL default '0',
			PRIMARY KEY (pid)
		);");
	}
	
	$db->delete_query('settings', "name='preparserpost'");
	$db->delete_query('settinggroups', "name='preparserpost_days'");
	$db->delete_query('settinggroups', "name='preparserpost_replies'");
	
	$db->insert_query("settinggroups", array(
		'name' => 'preparserpost', 
		'title' => 'PreParser Post Cache Settings', 
		'description' => '', 
		'disporder' => 99, 
		'isdefault' => 0
	));
	$gid = $db->insert_id();
	
	$db->insert_query('settings', array(
		'gid' => $gid,
		'name' => 'preparserpost_days',
		'title' => 'Keep Parsed Posts for x days',
		'description' => 'How long preparsed posts are stored in the database. (0 to disable)',
		'optionscode' => 'text',
		'value' => 30,
		'disporder' => 1
	));
	
	$db->insert_query('settings', array(
		'gid' => $gid,
		'name' => 'preparserpost_replies',
		'title' => 'Keep Parsed Threads with x replies',
		'description' => 'How many replies a thread should have for the posts in it to be preparsed. (0 to disable)',
		'optionscode' => 'text',
		'value' => 100,
		'disporder' => 2
	));
	
	if(!isset($parser))
	{
		require_once MYBB_ROOT.'inc/class_parser.php';
		$parser = new postParser;
	}
	
	// THIS MAKE TAKE A LONG TIME.
	$query = $db->query("
		SELECT p.pid, p.message, p.username, u.username as userusername, f.allowhtml, f.allowmycode, f.allowsmilies, f.allowimgcode, t.tid
		FROM ".TABLE_PREFIX."posts p
		LEFT JOIN ".TABLE_PREFIX."threads t ON (t.tid=p.tid)
		LEFT JOIN ".TABLE_PREFIX."forums f ON (f.fid=p.fid)
		LEFT JOIN ".TABLE_PREFIX."users u ON (u.uid=p.uid)
		WHERE t.replies >= 99 OR t.lastpost >= '".(time()-(60*60*24*30))."'
	");
	while($post = $db->fetch_array($query))
	{		
		$parser_options = array();
		$parser_options['allow_html'] = $post['allowhtml'];
		$parser_options['allow_mycode'] = $post['allowmycode'];
		$parser_options['allow_smilies'] = $post['allowsmilies'];
		$parser_options['allow_imgcode'] = $post['allowimgcode'];
		$parser_options['filter_badwords'] = 1;
		
		if(!$post['username'])
		{
			$post['username'] = $lang->guest;
		}
		
		if($post['userusername'])
		{
			$parser_options['me_username'] = $post['userusername'];
		}
		else
		{
			$parser_options['me_username'] = $post['username'];
		}
		
		$db->query("REPLACE INTO ".TABLE_PREFIX."parsedposts SET pid='{$post['pid']}', tid='{$post['tid']}', parsedpost='".$db->escape_string($parser->parse_message($post['message'], $parser_options))."'");
	}
}

function preparserpost_is_installed()
{
	global $db;
	
	if($db->table_exists('parsedposts'))
	{
		return true;
	}
	
	return false;
}

function preparserpost_activate()
{
}

function preparserpost_deactivate()
{
}

function preparserpost_uninstall()
{
	global $db;

	$db->drop_table("parsedposts");
	
	$db->delete_query('settings', "name='preparserpost'");
	$db->delete_query('settinggroups', "name='preparserpost_days'");
	$db->delete_query('settinggroups', "name='preparserpost_replies'");
}

function preparserpost_showthread()
{
	global $mybb;
	
	if($mybb->input['mode'] != "threaded")
	{
		$mybb->user['already_parsed'] = 1;
	}
}

function preparserpost_functions_post(&$post)
{
	global $mybb;
	
	if($mybb->input['mode'] == "threaded" || $mybb->input['ajax'])
	{
		return;
	}
	
	// Is this post not pre-parsed?
	if($post['parsedpost'] == "" && $post['message'] != "")
	{
		global $parser, $db, $forum, $lang;
		
		if(!isset($parser))
		{
			require_once MYBB_ROOT.'inc/class_parser.php';
			$parser = new postParser;
		}
			
		$parser_options = array();
		$parser_options['allow_html'] = $forum['allowhtml'];
		$parser_options['allow_mycode'] = $forum['allowmycode'];
		$parser_options['allow_smilies'] = $forum['allowsmilies'];
		$parser_options['allow_imgcode'] = $forum['allowimgcode'];
		$parser_options['filter_badwords'] = 1;
		
		if(!$post['username'])
		{
			$post['username'] = $lang->guest;
		}
		
		if($post['userusername'])
		{
			$parser_options['me_username'] = $post['userusername'];
		}
		else
		{
			$parser_options['me_username'] = $post['username'];
		}
		
		$post['parsedpost'] = $parser->parse_message($post['message'], $parser_options);
	}
		
	$post['message'] = $post['parsedpost'];

	get_post_attachments($post['pid'], $post);
}

function preparserpost_datahandler_post_update(&$post)
{
	global $parser, $mybb, $db;
	
	if(!isset($parser))
	{
		require_once MYBB_ROOT.'inc/class_parser.php';
		$parser = new postParser;
	}
	
	$query = $db->query("
		SELECT f.allowhtml,f.allowmycode,f.allowsmilies,f.allowimgcode,t.tid
		FROM ".TABLE_PREFIX."posts p
		LEFT JOIN ".TABLE_PREFIX."threads t ON (t.tid=p.tid)
		LEFT JOIN ".TABLE_PREFIX."forums f ON (f.fid=t.fid)
		WHERE p.pid='{$post->pid}'
		LIMIT 1
	");
	$forum = $db->fetch_array($query);
	
	$parser_options = array();
	$parser_options['allow_html'] = $forum['allowhtml'];
	$parser_options['allow_mycode'] = $forum['allowmycode'];
	$parser_options['allow_smilies'] = $forum['allowsmilies'];
	$parser_options['allow_imgcode'] = $forum['allowimgcode'];
	$parser_options['filter_badwords'] = 1;
	
	if(!$post->data['username'])
	{
		$post->data['username'] = $lang->guest;
	}

	$parser_options['me_username'] = $post->data['username'];
	
	$db->query("REPLACE INTO ".TABLE_PREFIX."parsedposts SET pid='{$post->pid}', tid='{$forum['tid']}', parsedpost='".$db->escape_string($parser->parse_message($post->data['message'], $parser_options))."'");
}

function preparserpost_datahandler_post_insert()
{
	global $parser, $mybb, $db, $posthandler, $lang;
	
	if(!isset($parser))
	{
		require_once MYBB_ROOT.'inc/class_parser.php';
		$parser = new postParser;
	}
	
	$query = $db->simple_select("forums", "allowhtml,allowmycode,allowsmilies,allowimgcode", "fid='{$posthandler->post_insert_data['fid']}'", array('limit' => 1));
	$forum = $db->fetch_array($query);
	
	$parser_options = array();
	$parser_options['allow_html'] = $forum['allowhtml'];
	$parser_options['allow_mycode'] = $forum['allowmycode'];
	$parser_options['allow_smilies'] = $forum['allowsmilies'];
	$parser_options['allow_imgcode'] = $forum['allowimgcode'];
	$parser_options['filter_badwords'] = 1;	

	if(!$posthandler->data['username'])
	{
		$posthandler->data['username'] = $lang->guest;
	}

	$parser_options['me_username'] = $posthandler->data['username'];
	
	$db->query("REPLACE INTO ".TABLE_PREFIX."parsedposts SET pid='{$posthandler->pid}', tid='{$posthandler->post_insert_data['tid']}', parsedpost='".$db->escape_string($parser->parse_message($posthandler->data['message'], $parser_options))."'");
}

?>